//---------------------------------------------------------------------------

#ifndef ScriptExecH
#define ScriptExecH
//---------------------------------------------------------------------------

#include <string>

class LuaState;
struct lua_State;



class ScriptExec
{
private:
	friend class LuaState;

	typedef void (__closure *CallbackAddOutputText)(const char* text);
	typedef void (__closure *CallbackCall)(const char* number);
	typedef void (__closure *CallbackHangup)(void);
	typedef std::string (__closure *CallbackGetDial)(void);
	typedef void (__closure *CallbackSetDial)(std::string number);
	typedef void (__closure *CallbackSwitchAudioSource)(std::string mod, std::string dev);	
	typedef void (__closure *CallbackSendDtmf)(const std::string& digits);
	typedef void (__closure *CallbackBlindTransfer)(const std::string& target);
	typedef int (__closure *CallbackGetCallState)(void);	///< get current call state, values as in Callback::ua_state_e
    typedef int (__closure *CallbackIsCallIncoming)(void);	///< check if call is incoming (check call direction)
	typedef std::string (__closure *CallbackGetCallPeer)(void);	///< get either caller or called number
	typedef int (__closure *CallbackGetStreamingState)(void);	///< get current streaming (paging) state, values as in Callback::paging_tx_state_e
	typedef std::string (__closure *CallbackGetInitialCallTarget)(void);
	typedef void (__closure *CallbackSetInitialCallTarget)(std::string number);

	CallbackAddOutputText onAddOutputText;
	CallbackCall onCall;
	CallbackHangup onHangup;
	CallbackGetDial onGetDial;
	CallbackSetDial onSetDial;
	CallbackSwitchAudioSource onSwitchAudioSource;
	CallbackSendDtmf onSendDtmf;
	CallbackBlindTransfer onBlindTransfer;
	CallbackGetCallState onGetCallState;
	CallbackIsCallIncoming onIsCallIncoming;
	CallbackGetCallPeer onGetCallPeer;
	CallbackGetStreamingState onGetStreamingState;
	CallbackGetInitialCallTarget onGetInitialCallTarget;
	CallbackSetInitialCallTarget onSetInitialCallTarget;

	static int LuaPrint(lua_State *L);
	static int LuaError( lua_State *L );
	static int l_ShowMessage( lua_State* L );
	static int l_Sleep(lua_State *L);
	static int l_Beep(lua_State *L);
	static int l_GetClipboardText(lua_State* L);	
	static int l_SetClipboardText(lua_State* L);
	static int l_Call(lua_State* L);
	static int l_Hangup(lua_State* L);
	static int l_GetDial(lua_State* L);
	static int l_SetDial(lua_State* L);
	static int l_SwitchAudioSource(lua_State* L);
	static int l_SendDtmf(lua_State* L);
	static int l_BlindTransfer(lua_State* L);
	static int l_GetCallState(lua_State* L);
	static int l_IsCallIncoming(lua_State* L);
	static int l_GetCallPeer(lua_State* L);
	static int l_GetStreamingState(lua_State* L);
	static int l_SetVariable(lua_State* L);
	static int l_GetVariable(lua_State* L);
	static int l_ClearVariable(lua_State* L);
	static int l_ClearAllVariables(lua_State* L);
	static int l_GetInitialCallTarget(lua_State* L);
	static int l_SetInitialCallTarget(lua_State* L);

	bool breakReq;
	bool running;
public:
	ScriptExec(
		CallbackAddOutputText onAddOutputText,
		CallbackCall onCall,
		CallbackHangup onHangup,
		CallbackGetDial onGetDial,
		CallbackSetDial onSetDial,
		CallbackSwitchAudioSource onSwitchAudioSource,
		CallbackSendDtmf onSendDtmf,
		CallbackBlindTransfer onBlindTransfer,
		CallbackGetCallState onGetCallState,
		CallbackIsCallIncoming onIsCallIncoming,
		CallbackGetCallPeer onGetCallPeer,
		CallbackGetStreamingState onGetStreamingState,
		CallbackGetInitialCallTarget onGetInitialCallTarget,
		CallbackSetInitialCallTarget onSetInitialCallTarget
		);
	~ScriptExec();
	void Run(const char* script);
	void Break(void);
	bool isRunning(void) {
		return running;
	}
};

#endif
